/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: MPEXlib.h,v $
 * Revision 1.1.2.5  1993/06/03  18:17:45  Pete_Nishimoto
 * 	remove ff/bfTextureMapIndex;add textureMapState
 * 	[1993/06/03  18:09:28  Pete_Nishimoto]
 *
 * Revision 1.1.2.4  1992/06/06  16:12:48  Thomas_True
 * 	"Syncpoint Submit"
 * 
 * Revision 1.1.3.2  92/06/06  16:12:35  Thomas_True
 * 	jun3dbg_r5 syncpoint
 * 
 * Revision 1.1.2.3  92/04/27  09:42:15  devbld_mlo
 * 	"Replacement of Incorrect Original"
 * 
 * Revision 1.1.3.2  92/04/27  09:42:01  devbld_mlo
 * 	Replacement of Incorrect Original
 * 
 * Revision 1.1.2.2  92/04/23  15:08:45  devbld_mlo
 * 	"Initial Submit"
 * 
 * Revision 1.1.1.2  92/04/23  15:08:28  devbld_mlo
 * 	Initial Checkin
 * 
 * $EndLog$
 */
/******************************************************************************/
/**                                                                          **/
/**  Copyright (c) 1989                                                      **/
/**  by DIGITAL Equipment Corporation, Maynard, Mass.                        **/
/**                                                                          **/
/**  This software is furnished under a license and may be used and  copied  **/
/**  only  in  accordance  with  the  terms  of  such  license and with the  **/
/**  inclusion of the above copyright notice.  This software or  any  other  **/
/**  copies  thereof may not be provided or otherwise made available to any  **/
/**  other person.  No title to and ownership of  the  software  is  hereby  **/
/**  transferred.                                                            **/
/**                                                                          **/
/**  The information in this software is subject to change  without  notice  **/
/**  and  should  not  be  construed  as  a commitment by DIGITAL Equipment  **/
/**  Corporation.                                                            **/
/**                                                                          **/
/**  DIGITAL assumes no responsibility for the use or  reliability  of  its  **/
/**  software on equipment which is not supplied by DIGITAL.                 **/
/**                                                                          **/
/******************************************************************************/

#ifndef MPEXLIB_H
#define MPEXLIB_H

#ifndef MPEX_H
#  include "MPEX.h"
#endif
#ifndef MPEXPROTO_H
#  include "MPEXproto.h"
#endif
#ifndef MPEXPROTOSTR_H
#  include "MPEXprotostr.h"
#endif

/*
 * Following are the typedefs used in the PEXlib interface. mpxlFooBar
 * definitions are always supersets of the mpexFooBar protocol definitions.
 * Note that most mpxlFooBar definitions are equivalent to the mpexFooBar
 * protocol definitions.  However, some of the definitions do have extra
 * fields. See documentation in PEXlib.h for more details. 
 */

/*
 *  Definitions for output commands
 */
typedef mpexAnnoPixmap                  mpxlAnnoPixmap;
typedef mpexArc                         mpxlArc;
typedef mpexCircle                      mpxlCircle;
typedef mpexIndexedPolygon              mpxlIndexedPolygon;
typedef mpexEllipticalArc2D		mpxlEllipticalArc2D;
typedef mpexEllipticalArc3D		mpxlEllipticalArc3D;
typedef mpexEllipse2D			mpxlEllipse2D;
typedef mpexEllipse3D			mpxlEllipse3D;
typedef mpexGridRectangular             mpxlGridRectangular;
typedef mpexGridRadial                  mpxlGridRadial;
typedef mpexEchoIndex                   mpxlEchoIndex;
typedef mpexHighlightIndex              mpxlHighlightIndex;
typedef mpexTextSkew			mpxlTextSkew;
typedef mpexAnnotationTextSkew		mpxlAnnotationTextSkew;
typedef mpexReferToStructure            mpxlReferToStructure;
typedef mpexTextureMapIndex             mpxlTextureMapIndex;
typedef mpexTextureMapState             mpxlTextureMapState;
typedef mpexNoop                        mpxlNoop;



/*
 *  Definitions for lookup table entries
 */
typedef struct
{
    pxlEnumTypeIndex    method B16;
    CARD16              pad B16;
    pxlColourSpecifier  colour;
} mpxlHighlightEntry;

typedef struct
{
    pxlEnumTypeIndex    method B16;
    CARD16              pad B16;
    pxlColourSpecifier  colour;
} mpxlEchoEntry;

typedef mpexPixmapEntry mpxlPixmapEntry;
typedef mpexTextureEntry mpxlTextureEntry;

/* 
 * pick path returned by pick all 
 */
typedef struct mpxlPickAllPath
{
    int                 pathLength;
    pxlPickPath         *elementRefs;
} mpxlPickAllPath;

/* 
 * Definition for MPEX Pipeline Context Attributes.  Defines are in MPEX.h. 
 */

typedef struct mpxlPCAttributes
{
    unsigned short	highlightIndex;
    unsigned short	echoIndex;
    float		textSkew;
    float		annotationTextSkew;
    unsigned short      textureMapIndex;
    unsigned short      textureMapState;
} mpxlPCAttributes;

/* macro for setting bits in a new PC value mask */

#define MPEX_SetPCAttrMaskBit(mask, attrNum) \
    mask |= 1L << ( ((attrNum)) & 0x1F)


/* 
 * Definition for MPEX Renderer Attributes.  Defines are in MPEX.h. 
 */

typedef struct mpxlRendererAttributes
{
    short		logicalOp;
    short		overlapMode;
    unsigned long	planeMask;
    unsigned long	backgroundPixel;
    unsigned char	clearI;
    unsigned char	clearZ;
    unsigned char       errorCheck;
    unsigned char	aaSwitch;
    unsigned char	echoSwitch;
    unsigned char	cullSwitch;
    pxlLookupTable	highlightTable;
    pxlLookupTable	echoTable;
    pxlLookupTable	pixmapTable;
    pxlLookupTable      textureTable;
    pxlDeviceCoord2D	pickPosition;
    float		pickDistance;
    pxlNameSet		pickInclusion;
    pxlNameSet		pickExclusion;
    unsigned long       pickMaxHits;
    pxlStructurePath	pickStartPath;
} mpxlRendererAttributes;


/* Event Structures */

/* The first several items in each event structure must be as they are or
   toolkits will break */

typedef struct mpxlMaxHitsReachedEvent
{
    int 		type;		/* type of event, MPEXMaxHitsReached */
    unsigned long	serial;		/* # of last req processed by server */
    Bool		send_event;	/* True if came from SendEvent req */
    Display		*display;	/* Display the event was read from */
    pxlRenderer		rdr;		/* renderer doing the pick */
} mpxlMaxHitsReachedEvent;

#endif /* MPEXLIB_H */
