#ifndef		__OBJECTION_HXX
#define		__OBJECTION_HXX

//
//
/*  Copyright Digital Equipment Corporation 1990, 1995. All rights reserved.*/
//
/*  Restricted Rights: Use, duplication, or disclosure by the U.S.*/
/*  Government is subject to restrictions as set forth in subparagraph*/
/*  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR*/
/*  52.227-14 Alt. III, as applicable.*/
//
/*  This software is proprietary to and embodies the confidential*/
/*  technology of Digital Equipment Corporation. Possession, use, or*/
/*  copying of this software and media is authorized only pursuant to a*/
/*  valid written license from Digital or an authorized sublicensor.*/
//
//

/*++*/
/*  FACILITY:	DEC C++ Class Library*/
//
/*  SUBSYSTEM:	Objection (primordial exception handling)*/
//
/*  MODULE:	objection*/
//
/*  INFORMATION HIDDEN:*/
//
/*	Representation of an Objection.*/
/*	Algorithms for operations on Objections.*/
//
/*  MODULE DESCRIPTION:*/
//
/*	This module implements the representation of Objections and defines*/
/*	the functions which operate on them.*/
//
/*--*/



//
//
/*  INCLUDE FILES*/
//
//

#include <cxxl.hxx>

extern "C"
{
#include <stddef.h>
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

//
//
/*  LINKAGE*/
//
//

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

//
//
/*  TYPE DEFINITIONS*/
//
//

typedef int Objection_action(const char *);

class Objection

/*++*/
/*  CLASS DESCRIPTION:*/
//
/*	An Objection is information about a potential error condition which*/
/*	a program can encounter.  An Objection can contain a pointer to a*/
/*	default error handling procedure and a current error handling procedure.*/
/*	After raising an objection, depending on the error handling procedures,*/
/*	execution may resume.*/
/*--*/

{
private:
    Objection_action *action;		/* Pointer to the current error handler*/
    Objection_action *default_action;	/* Pointer to the default error handler*/
    int ignore_objection;		/* non-0: Ignore the objection;*/
					/* do not invoke any error handlers.*/
					/* 0: Do not ignore the objection;*/
					/* invoke error handlers.*/

public:

    Objection();
    Objection(Objection_action *new_action);
    int raise(const char *msg ="");
    Objection_action *appoint(Objection_action *new_action);
    Objection_action *appoint();
    Objection_action *ignore();
};

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
/* End of objection.hxx*/
