#ifndef	__STOPWATCH_HXX
#define	__STOPWATCH_HXX

//
//
/*  Copyright Digital Equipment Corporation 1990, 1995. All rights reserved.*/
//
/*  Restricted Rights: Use, duplication, or disclosure by the U.S.*/
/*  Government is subject to restrictions as set forth in subparagraph*/
/*  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR*/
/*  52.227-14 Alt. III, as applicable.*/
//
/*  This software is proprietary to and embodies the confidential*/
/*  technology of Digital Equipment Corporation. Possession, use, or*/
/*  copying of this software and media is authorized only pursuant to a*/
/*  valid written license from Digital or an authorized sublicensor.*/
//
//

/*++*/
/*  FACILITY:	DEC C++ Class Library*/
//
/*  SUBSYSTEM:	The Stopwatch package.*/
//
/*  MODULE:	Include file for the Stopwatch package.*/
//
/*  MODULE DESCRIPTION:*/
//
/*      The Stopwatch package is used for timing measurements in user programs.*/
//
/*  DESIGN ISSUES:*/
//
/*      The Stopwatch package included in DEC C++ Class Library fully conforms*/
/*      to the description in "UNIX(R) System V AT&T Standard Library Extension*/
/*      Release 1.0, Manual and Tutorials, Select Code 309-525".*/
//
/*--*/


/*++*/
/*  INCLUDE FILES*/
/*--*/

#include <cxxl.hxx>

extern "C"
{
#include <time.h>
#if defined(__unix) || defined(__VMS_POSIX)
#   include <sys/times.h>
#endif
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment save
#   pragma __environment header_defaults
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model __save
#      pragma __extern_prefix __save
#      pragma __member_alignment __save
#   endif
#endif

/*++*/
/*  LINKAGE*/
/*--*/

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_model __relaxed_refdef
#   pragma __extern_prefix __CXXL_PLAIN_FUNC_LINKAGE
#   if defined __VAX
#       pragma __nomember_alignment
#   else
#       pragma __member_alignment
#   endif
#endif

/*++*/
/*  TYPE DEFINITIONS*/
/*--*/

#if defined(__unix) || defined(__VMS_POSIX)
    typedef struct tms tms_t;
#else
#   if __VMS_VER >= 70000000
      typedef struct tms tms_t;
#   else 
#     define tms	tbuffer
#     define tms_t	tbuffer_t
#     define tms_utime	proc_user_time
#     define tms_stime	proc_system_time
#     define tms_cutime	child_user_time
#     define tms_cstime	child_system_time
#   endif
#endif


/*++*/
/*  STATIC DATA*/
/*--*/

/* The default resolution is 1/60 second*/
/* (This is usually defined in <time.h>)*/

#ifndef CLK_TCK
#   define CLK_TCK 60
#endif

/* The clock resolution in seconds*/

static const float cxxl_resolution = 1.0 / CLK_TCK;

class Stopwatch

/*++*/
/*  FUNCTIONAL DESCRIPTION:*/
//
/*	A Stopwatch is a device for the measurement of time intervals useful to*/
/*	programmers:  user time, system time, and real time.  The distinction*/
/*	between user time and system time is only meaningful on ULTRIX.  On VMS,*/
/*	consistently with the behavior of the VAX C RTL routine times(), the*/
/*	total accumulated CPU time for the process is returned as user time,*/
/*	and system time is always returned as zero.*/
/*--*/

{
public:

#if defined(__VMS) && defined(__DECCXX)
#   pragma __extern_prefix __CXXL_FLOAT_FUNC_LINKAGE
#endif

    /* Constructors*/
			Stopwatch();

    /* Setting operations*/
    void		start();
    void		stop();
    void		reset();

    /* Reading operations*/
    inline int		status() const;

    double		system();
    double		user();
    double		real();

    /* Miscellaneous operations*/
    inline double	resolution();

private:
    tms_t	   	cxxl_time_buf;
    int			cxxl_status;
    time_t		cxxl_init_system_time;
    time_t		cxxl_init_user_time;
    time_t		cxxl_init_real_time;
    double		cxxl_system_time;
    double		cxxl_user_time;
    double		cxxl_real_time;    
};

inline int Stopwatch::status(
    ) const

/*++*/
/*  FUNCTIONAL DESCRIPTION:*/
//
/*      Returns the status of the Stopwatch.*/
//
/*  RETURN VALUE:*/
//
/*	1 if the Stopwatch is running, and 0 otherwise.*/
/*--*/

{
    return cxxl_status;
}

inline double Stopwatch::resolution(
    )

/*++*/
/*  FUNCTIONAL DESCRIPTION:*/
//
/*	Returns the (system-dependent) resolution of the Stopwatch in seconds.*/
/*	On ULTRIX systems, 1/60 is typical, while 1/100 is the resolution*/
/*	on VMS systems.  Note that the value of resolution pertains only to*/
/*	system and user time.  Values of real (clock) time are always expressed*/
/*	in whole seconds.*/
//
/*  RETURN VALUE:*/
//
/*      The fixed, system-dependent resolution of the Stopwatch in seconds.*/
/*--*/

{
    return (double)cxxl_resolution;
}

#ifdef __PRAGMA_ENVIRONMENT
#   pragma __environment restore
#else
#   if defined(__VMS) && defined(__DECCXX)
#      pragma __extern_model  __restore
#      pragma __extern_prefix __restore
#      pragma __member_alignment __restore
#   endif
#endif

#endif
/* End of stopwatch.hxx*/
